"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const tool_lib_1 = require("@kettil/tool-lib");
const access_1 = __importDefault(require("../../lib/cmd/access"));
const readFile_1 = __importDefault(require("../../lib/cmd/readFile"));
const writeFile_1 = __importDefault(require("../../lib/cmd/writeFile"));
const update = (config, values) => {
    if (tool_lib_1.objectKeys(values).length === 0) {
        return config;
    }
    return Object.assign(tool_lib_1.isObject(config) ? config : {}, values);
};
const updatePeerDependencies = (config, values) => {
    const peerDependencies = {};
    const packages = config.devDependencies;
    if (!tool_lib_1.isObject(packages)) {
        throw new Error('In the package.json is the devDependencies empty');
    }
    values.forEach((value) => {
        const version = packages[value];
        if (typeof version !== 'string') {
            throw new TypeError(`The peerDependency "${value}" is not found in the devDependencies`);
        }
        peerDependencies[value] = version;
    });
    return update(config.peerDependencies, peerDependencies);
};
const npmPackage = async ({ cwd, log }, { settings: { bin, scripts, peerDependencies, ...settings } }) => {
    const path = path_1.join(cwd, 'package.json');
    const isExists = await access_1.default(path);
    if (!isExists) {
        throw new Error('package.json is not found');
    }
    const config = await readFile_1.default(path, true);
    Object.assign(config, settings);
    // Update the bin part
    if (bin) {
        config.bin = update(config.bin, bin);
    }
    // Update the scripts part
    if (scripts) {
        config.scripts = update(config.scripts, scripts);
    }
    // Update the peerDependencies part
    if (peerDependencies && peerDependencies.length > 0) {
        config.peerDependencies = updatePeerDependencies(config, peerDependencies);
    }
    log.info('Extend the package.json');
    await writeFile_1.default(path, config);
};
exports.default = npmPackage;
